/*! ********************************************
 * EFXStereoReverb.as
 * Copyright 2019 masahiro takamoto (MotChan)
 * Distributed under the MIT license. 
 * see -> sora/license.txt
 ******************************************** */
package sora.graybox.efx
{
import sora.graybox.GrayBox;
import sora.graybox.ProcessData;
import sora.graybox.IGrayBoxModule;
public class EFXStereoReverb extends EffectorBase implements IGrayBoxModule
{
// EFXStereoReverb --- 
	// Ring Buffer
	private var _ringBufferSizeBits:int = 15;
	private var _ringBufferSize:int = 1 << _ringBufferSizeBits;
	private var _ringBufferMask:int = _ringBufferSize - 1;
	private var _delayBuffer1:Vector.<Number>;
	private var _delayBuffer2:Vector.<Number>;
	// 
	private var _sampleRate:Number = 44100.0;
	// Delay
	private var _wp :int = 0;  // WritePoint
	// Delay 1
	private var _feedBack1  :Number = 0.5; // 0 ~ 1.0
	private var _rp1        :int = 0;     // ReadPoint
	private var _delay1     :int = 0;     // 0 ~ _ringBufferMask
	private var _delayTime1 :Number = 0.0;  // [Sec]
	// Delay 2
	private var _feedBack2  :Number = 0.3;
	private var _rp2        :int = 0;
	private var _delay2     :int = 0;
	private var _delayTime2 :Number = 0.0;  // [Sec]
// getter, setter
	public function getRingBufferSize():int { return _ringBufferSize; }
	public function getSampleRate():Number { return _sampleRate; }
	public function setSampleRate(rate:Number):void
	{
		_sampleRate = rate;
		_delayTime1 = _delay1 / _sampleRate;
		_delayTime2 = _delay2 / _sampleRate;
	}
	public function getDelayFeedBackLevel1():Number { return _feedBack1; }
	public function getDelayFeedBackLevel2():Number { return _feedBack2; }
	public function setDelayFeedBackLevel1(val:Number):void { _feedBack1 = val; }
	public function setDelayFeedBackLevel2(val:Number):void { _feedBack2 = val; }
	public function getDelayTime1():Number { return _delayTime1; }
	public function getDelayTime2():Number { return _delayTime2; }
	public function setDelayTime1(val:Number):Number
	{
		var _frame:int = val * _sampleRate;
		if (_frame > _ringBufferMask)_frame = _ringBufferMask;
		_delay1 = _frame;
		_rp1 = _wp + (_ringBufferMask - _delay1) & _ringBufferMask;
		return _delayTime1 = _frame / _sampleRate;
	}
	public function setDelayTime2(val:Number):Number
	{
		var _frame:int = val * _sampleRate;
		if (_frame > _ringBufferMask)_frame = _ringBufferMask;
		_delay2 = _frame;
		_rp2 = _wp + (_ringBufferMask - _delay2) & _ringBufferMask;
		return _delayTime2 = _frame / _sampleRate;
	}
	public function getDelayFrame1():int { return _delay1; }
	public function getDelayFrame2():int { return _delay2; }
	public function setDelayFrame1(val:int):void
	{ 
		_delay1 = val;
		if (_delay1 > _ringBufferMask)_delay1 = _ringBufferMask;
		_rp1 = _wp + (_ringBufferMask - _delay1) & _ringBufferMask;
		_delayTime1 = _delay1 / _sampleRate;
	}
	public function setDelayFrame2(val:int):void
	{ 
		_delay2 = val;
		if (_delay2 > _ringBufferMask)_delay2 = _ringBufferMask;
		_rp2 = _wp + (_ringBufferMask - _delay2) & _ringBufferMask;
		_delayTime2 = _delay2 / _sampleRate;
	}
	public function setEffectParam(mix:Number = 0.3, 
		delayTime1:Number = 0.1, feedBackLevel1:Number = 0.5,
		delayTime2:Number = 0.25, feedBackLevel2:Number = 0.2
		):void
	{
		_mix = mix;
		_antiMix = 1.0 - _mix;
		setDelayTime1(delayTime1);
		setDelayTime2(delayTime2);
		_feedBack1 = feedBackLevel1;
		_feedBack2 = feedBackLevel2;
	}
// Constructor
	public function EFXStereoReverb(bufferSizeBits:int = 15):void
	{
		super();
		// bufferSize
		if (bufferSizeBits > 20) bufferSizeBits = 20;
		else if (bufferSizeBits < 4) bufferSizeBits = 4;
		_ringBufferSizeBits = bufferSizeBits;
		_ringBufferSize = 1 << _ringBufferSizeBits;
		_ringBufferMask = _ringBufferSize - 1;
		_delayBuffer1 = new Vector.<Number>(_ringBufferSize, true);
		_delayBuffer2 = new Vector.<Number>(_ringBufferSize, true);
		// reset
		clearBuffer();
		setEffectParam();
	}
// Base - EffectorBase ---
	override public function applyEffect(outL:Vector.<Number>, outR:Vector.<Number>, startFrame:int, sampleFrames:int):void
	{
		var _v:Number, i:int = startFrame,  
			_endFrame:int = i + sampleFrames;
		for ( ; i < _endFrame; i++,
			_rp1 = _rp1 + 1 & _ringBufferMask,
			_rp2 = _rp2 + 1 & _ringBufferMask,
			_wp  = _wp  + 1 & _ringBufferMask)
		{
		// Channel 1 --->
			_v  = _delayBuffer1[_rp1] * _feedBack1;
			_v += _delayBuffer1[_rp2] * _feedBack2;
			_delayBuffer1[_wp] = outL[i] + _v;
			outL[i] = outL[i] * _antiMix + _v * _mix;	// outout
		// Channel 2 --->
			_v  = _delayBuffer2[_rp1] * _feedBack1;
			_v += _delayBuffer2[_rp2] * _feedBack2;
			_delayBuffer2[_wp] = outR[i] + _v;
			outR[i] = outR[i] * _antiMix + _v * _mix;	// outout
		}
	}
	override public function resetParameters():void
	{
		setEffectParam();
	}
	override public function clearBuffer():void
	{
		for (var i:int = 0; i < _ringBufferSize; i++)_delayBuffer1[i] = _delayBuffer2[i] = 0.0;
	}
	override public function initialize():void
	{
		clearBuffer();
		setEffectParam();
	}
// Interface - IfGrayBoxModule -----
	public function updateSampleRate(sampleRate:Number):void{ setSampleRate(sampleRate); }
	public function updateAudioChannels(ch:int):void{}
	public function updateProcessBlockSize(size:int):void{}
	
	public function resetModule():void{ initialize(); }
	public function initiModule():void{ initialize(); }
	public function connectModule(host:GrayBox):void{ clearBuffer(); setSampleRate(host.getSampleRate()); }
	public function disconnectModule():void{ clearBuffer(); }
	public function updateBypassStatus(bypass:Boolean):void{ clearBuffer(); }
	
	public function startProcess():void{ clearBuffer(); }
	public function stopProcess():void{ clearBuffer(); }
	public function suspendProcess():void{ clearBuffer(); }
	public function resumeProcess():void{ clearBuffer(); }
	public function process(procData:ProcessData):void
	{
		// inline --- applyEffect();
		var _v:Number, i:int = procData.offsetFrames,  
			_endFrame:int = i + procData.sampleFrames,
			_outL:Vector.<Number> = procData.audioOut[0],
			_outR:Vector.<Number> = procData.audioOut[1];
		for ( ; i < _endFrame; i++,
			_rp1 = _rp1 + 1 & _ringBufferMask,
			_rp2 = _rp2 + 1 & _ringBufferMask,
			_wp  = _wp  + 1 & _ringBufferMask)
		{
		// Channel 1 --->
			_v  = _delayBuffer1[_rp1] * _feedBack1;
			_v += _delayBuffer1[_rp2] * _feedBack2;
			_delayBuffer1[_wp] = _outL[i] + _v;
			_outL[i] = _outL[i] * _antiMix + _v * _mix;	// outout
		// Channel 2 --->
			_v  = _delayBuffer2[_rp1] * _feedBack1;
			_v += _delayBuffer2[_rp2] * _feedBack2;
			_delayBuffer2[_wp] = _outR[i] + _v;
			_outR[i] = _outR[i] * _antiMix + _v * _mix;	// outout
		}
	}
}
}