/*! ********************************************
 * TonePresetPCM.as
 * Copyright 2019 masahiro takamoto (MotChan)
 * Distributed under the MIT license. 
 * see -> sora/license.txt
 ******************************************** */
package sora.graybox.synth
{
/*
	- Parameters Notes -
	
	[toneName]
	  16 Chars (ASCII Code)
	  
	[lfoFreq]
	  LFO Frequency [ 0 ~ 16383 ]
	  ( Frequency = lfoFreq * 0.001953125 )
	      0 ->  0 Hz
	    128 ->  0.25 Hz
	    256 ->  0.50 Hz
	    512 ->  1.00 Hz
	   1024 ->  2.00 Hz
	  16383 -> 31.998046875 Hz
	
	[strc]
	  Structure Number[ 0 ~ 3 ]
	  0 -> Instrument (LFO Off) 
	  1 -> Instrument (LFO PM) 
	  2 -> Instrument (LFO AM) 
	  
	[tune]
	  Tune [0(-1 octave) ~ 8192(+-0) ~ 16383(+1 octave)]
	  8192 / 1 octave
	  
	[lms]
	  LFO Modulation Depth Senstivity [0 ~ 15]
	  Modulation Depth = Midi control Change 1[0~127] * lms[0 ~ 15]
	  
	[smplNum]
	  SampleData Number [0 ~ 127]
	  
	[tl]
	  Total Level [0 ~ 127]
	  0 -> 0dB
	  4 -> -3dB
	  8 -> -6dB
	  16 -> -12dB
	  127 -> -96dB
	  
	[ar]
	  Attack Rate [0 ~ 127]
	    0 -> AttackTime 256.0Sec / 1
	    8 -> AttackTime 256.0Sec / 2
	   16 -> AttackTime 256.0Sec / 4
	   24 -> AttackTime 256.0Sec / 8
	   32 -> AttackTime 256.0Sec / 16
	  128 -> AttackTime 256.0Sec / 65536
	  
	[dr]
	  Decay Rate [0 ~ 127]
	    0 -> DecayTime 256.0Sec / 1
	    8 -> DecayTime 256.0Sec / 2
	   16 -> DecayTime 256.0Sec / 4
	   24 -> DecayTime 256.0Sec / 8
	   32 -> DecayTime 256.0Sec / 16
	  128 -> DecayTime 256.0Sec / 65536
	  
	[sl]
	  Sustain Level [0 ~ 127]
	  0 -> 0dB
	  4 -> -3dB
	  8 -> -6dB
	  16 -> -12dB
	  127 -> -96dB
	  
	[rr]
	  Release Rate [0 ~ 127]
	    0 -> DecayTime 256.0Sec / 1
	    8 -> DecayTime 256.0Sec / 2
	   16 -> DecayTime 256.0Sec / 4
	   24 -> DecayTime 256.0Sec / 8
	   32 -> DecayTime 256.0Sec / 16
	  128 -> DecayTime 256.0Sec / 65536
	  
	[kf]
	  Key Follow Sensitivity [0 - 3] (amp EG)
	  0 -> 48 Key / 1 octave
	  1 -> 24 Key / 1 octave
	  2 -> 12 Key / 1 octave
	  3 ->  6 Key / 1 octave
	  
	[kfStart]
	  StartKey [0 - 127] (Key Follow)
*/
public class TonePresetPCM
{
// Inastance -----
	public var toneName	:String = "  default preset";
	public var lfoFreq	:int = 2048; // 0 - 16383   LFO - Phase Update Timer Parameter
	public var strc		:int = 0;    // 0 - 3
	public var tune		:int = 8192; // 0 - 16383   center 8192, octave = 8192
	public var lms		:int = 0;    // 0 - 15      LFO - Modulation Depth Senstivity
	public var smplNum	:int = 0;    // 0 - 127
	public var tl		:int = 0;    // 0 - 127
	public var ar		:int = 126;  // 0 - 127
	public var dr		:int = 100;  // 0 - 127
	public var sl		:int = 0;    // 0 - 127
	public var rr		:int = 120;  // 0 - 127
	public var kf		:int = 0;    // 0 - 3       Key Follow Sensitivity
	public var kfStart	:int = 48;   // 0 - 127     Key Number - Start Key Follow
// Method
	public function copy(preset:TonePresetPCM):void
	{
		toneName = preset.toneName;
		strc = preset.strc;
		smplNum = preset.smplNum;
		tune = preset.tune;
		
		kf = preset.kf;
		kfStart = preset.kfStart;
		
		tl = preset.tl;
		ar = preset.ar;
		dr = preset.dr;
		sl = preset.sl;
		rr = preset.rr;
		
		lms = preset.lms;
		lfoFreq = preset.lfoFreq;
	}
	public function reset():void
	{
		toneName = "  default preset";
		lfoFreq = 2048;
		strc = 0;
		tune = 8192;
		lms = 0;
		smplNum = 0;
		tl = 0;
		ar = 126;
		dr = 100;
		sl = 0;
		rr = 120;
		kf = 0;
		kfStart	= 48;
	}
// Constructor
	public function TonePresetPCM():void{}
}
}