/*! ********************************************
 * TonePresetWT.as
 * Copyright 2019 masahiro takamoto (MotChan)
 * Distributed under the MIT license. 
 * see -> sora/license.txt
 ******************************************** */
package sora.graybox.synth
{
/*
	- Parameters Notes -
	
	[toneName]
	  16 Chars (ASCII Code)
	[key] * This parameter is enabled when the VoicePart is in mode 2
	  Key Pitch [NoteNumber(0~127)]
	
	[pan] * Enabled -> VoicePart mode 2
	  Panpot [ 0(L) ~ 64(center) ~ 127(R) ]
	
	[lfoFreq] * Enabled -> Strcture Number 0 ~ 2
	  LFO Frequency [ 0 ~ 16383 ]
	  ( Frequency = lfoFreq * 0.001953125 )
	      0 ->  0 Hz
	    128 ->  0.25 Hz
	    256 ->  0.50 Hz
	    512 ->  1.00 Hz
	   1024 ->  2.00 Hz
	  16383 -> 31.998046875 Hz
	
	[strc]
	  Structure Number[ 0 ~ 3 ]
	  0 -> Instrument (LFO Off) 
	  1 -> Instrument (LFO PM) 
	  2 -> Instrument (LFO AM) 
	  3 -> Noise Generator
	  
	[tune]
	  Tune [0(-1 octave) ~ 8192(+-0) ~ 16383(+1 octave)]
	  8192 / 1 octave
	  
	[lms] * Enabled -> Strcture Number 0 ~ 2
	  LFO Modulation Depth Senstivity [0 ~ 15]
	  Modulation Depth = Midi control Change 1[0~127] * lms[0 ~ 15]
	  
	[wave] * Enabled -> Strcture Number 0 ~ 2
	  WaveTable Index Nuber [0 ~ 127]
	  
	[tl]
	  Total Level [0 ~ 127]
	  0 -> 0dB
	  4 -> -3dB
	  8 -> -6dB
	  16 -> -12dB
	  127 -> -96dB
	  
	[ar]
	  Attack Rate [0 ~ 127]
	    0 -> AttackTime 256.0Sec / 1
	    8 -> AttackTime 256.0Sec / 2
	   16 -> AttackTime 256.0Sec / 4
	   24 -> AttackTime 256.0Sec / 8
	   32 -> AttackTime 256.0Sec / 16
	  128 -> AttackTime 256.0Sec / 65536
	  
	[dr]
	  Decay Rate [0 ~ 127]
	    0 -> DecayTime 256.0Sec / 1
	    8 -> DecayTime 256.0Sec / 2
	   16 -> DecayTime 256.0Sec / 4
	   24 -> DecayTime 256.0Sec / 8
	   32 -> DecayTime 256.0Sec / 16
	  128 -> DecayTime 256.0Sec / 65536
	  
	[sl]
	  Sustain Level [0 ~ 127]
	  0 -> 0dB
	  4 -> -3dB
	  8 -> -6dB
	  16 -> -12dB
	  127 -> -96dB
	  
	[rr]
	  Release Rate [0 ~ 127]
	    0 -> DecayTime 256.0Sec / 1
	    8 -> DecayTime 256.0Sec / 2
	   16 -> DecayTime 256.0Sec / 4
	   24 -> DecayTime 256.0Sec / 8
	   32 -> DecayTime 256.0Sec / 16
	  128 -> DecayTime 256.0Sec / 65536
	  
	[kf] * Enabled -> VoicePart mode 1
	  Key Follow Sensitivity [0 - 3] (amp EG)
	  0 -> 48 Key / 1 octave
	  1 -> 24 Key / 1 octave
	  2 -> 12 Key / 1 octave
	  3 ->  6 Key / 1 octave
	  
	[kfStart] * Enabled -> VoicePart mode 1
	  StartKey [0 - 127] (Key Follow)
	
	---------------------------------
	VoicePart Mode  | 1 | 2 | 1 | 2 |
	Strcture Number | 0 ~ 2 |   3   |
	---------------------------------
	[key]           | x | o | x | o |
	[pan]           | x | o | x | o |
	[lfoFreq]       | o | o | x | x | 
	[strc]          | o | o | o | o | 
	[tune]          | o | o | o | o | 
	[lms]           | o | o | x | x | 
	[wave]          | o | o | x | x | 
	[tl]            | o | o | o | o | 
	[ar]            | o | o | o | o | 
	[dr]            | o | o | o | o | 
	[sl]            | o | o | o | o | 
	[rr]            | o | o | o | o | 
	[kf]            | o | x | o | x | 
	[kfStart]       | o | x | o | x | 
	---------------------------------
	o = parameter is enable.
	x = parameter is disable.
	---------------------------------
	VoicePart Mode 0 -> Mute (channel mute)
	VoicePart Mode 1 -> Instrument Mode (Assign TonePreset[  0 ~ 127] to NoteNumber[0 ~ 127] )
	VoicePart Mode 2 -> Percussion Mode (Assign TonePreset[128 ~ 255] to NoteNumber[0 ~ 127] )
	VoicePart Mode 3 -> Sampler Mode
	---------------------------------
	Strcture Number 0 -> Instrument (LFO Off) 
	Strcture Number 1 -> Instrument (LFO PM) 
	Strcture Number 2 -> Instrument (LFO AM) 
	Strcture Number 3 -> Noise Generator
	---------------------------------
*/
public class TonePresetWT
{
// Inastance -----
	public var toneName	:String = "  default preset";
	public var key		:int = 64;   // 0 - 127     *Percussion
	public var pan		:int = 64;   // 0 - 127     *Percussion
	public var lfoFreq	:int = 2048; // 0 - 16383   LFO - Phase Update Timer Parameter
	public var strc		:int = 0;    // 0 - 3       Structure Number
	public var tune		:int = 8192; // 0 - 16383   center 8192, octave = 8192
	public var lms		:int = 0;    // 0 - 15      LFO - Modulation Depth Senstivity
	public var wave		:int = 0;    // 0 - 127     WaveTable ID
	public var tl		:int = 0;    // 0 - 127     Total Level
	public var ar		:int = 127;  // 0 - 127     Attack Rate
	public var dr		:int = 100;  // 0 - 127     Decay Rate
	public var sl		:int = 0;    // 0 - 127     Sustain Level
	public var rr		:int = 120;  // 0 - 127     Release Rate
	public var kf		:int = 0;    // 0 - 3       Key Follow Sensitivity         *Instrument 
	public var kfStart	:int = 48;   // 0 - 127     Key Number - Start Key Follow  *Instrument
// Method
	public function copy(preset:TonePresetWT):void
	{
		toneName = preset.toneName;
		strc = preset.strc;
		wave = preset.wave;
		tune = preset.tune;
		
		kf = preset.kf;
		kfStart = preset.kfStart;
		
		tl = preset.tl;
		ar = preset.ar;
		dr = preset.dr;
		sl = preset.sl;
		rr = preset.rr;
		
		lms = preset.lms;
		lfoFreq = preset.lfoFreq;
		
		key = preset.key;
		pan = preset.pan;
	}
	public function reset():void
	{
		toneName = "  default preset";
		key = 64;
		pan = 64;
		lfoFreq = 2048;
		strc = 0;
		tune = 8192;
		lms = 0;
		wave = 0;
		tl = 0;
		ar = 127;
		dr = 100;
		sl = 0;
		rr = 120;
		kf = 0;
		kfStart	= 48;
	}
// Constructor
	public function TonePresetWT():void{}
}
}