/*! ********************************************
 * VoiceUnitFM4.as
 * Copyright 2019 masahiro takamoto (MotChan)
 * Distributed under the MIT license. 
 * see -> sora/license.txt
 ******************************************** */
package sora.graybox.synth 
{
internal final class VoiceUnitFM4 extends VoiceUnitBase
{
// Tables
	// Log2 Volume Table
	private var _LOG2V_TABLE   :Vector.<Number> = null;
	private var _LOG2V_TABLE_I :Vector.<int> = null;
	// Pow2 Table
	private const _POW2T_OCTV_BITS :int = SynthData.POW2T_OCTV_BITS;
	private const _POW2T_OCTV_MASK :int = SynthData.POW2T_OCTV_MASK;
	private var _POW2_TABLE      :Vector.<int> = null;
	private var _POW2T_RS_TABLE  :Vector.<int> = null;
	private var _POW2T_KEY_TABLE :Vector.<int> = null;
	// EG
	private const _EG_PRAM_OCTV_LS :int = SynthData.POW2T_OCTV_BITS - 3;
	private const _EG_CNT_FP       :int = SynthData.EG_CNT_FP;
	private const _EG_MAX_CNT      :int = SynthData.EG_MAX_CNT;
	private const _EG_CNT_ENV_ATK  :int = SynthData.EG_CNT_ENV_ATK;
	private var _EG_TOTAL_LEVEL_TABLE :Vector.<int> = null;
	private var _EG_LEVEL_TABLE_UP    :Vector.<int> = null;
	private var _EG_LEVEL_TABLE_DW    :Vector.<int> = null;
	private var _EG_KEY_TABLE         :Vector.<int> = null;
	private var _EG_POW2T_RS_TABLE    :Vector.<int> = null;
	private var _EG_CNT_GATE_TABLE    :Vector.<int> = null;
	// Wave
	private const _WAVE_TABLE_MASK :int = SynthesizerFM2.WAVE_TABLE_SIZE - 1;
	// PG
	private const _PG_PHASE_FP      :int = SynthesizerFM2.PG_PHASE_FP;
	private const _PG_PHASE_FP_MASK :int = (1 << _PG_PHASE_FP) - 1;  // after the fixed-point
	private const _PG_PHASE_MASK    :int = (SynthesizerFM2.WAVE_TABLE_SIZE << _PG_PHASE_FP) - 1;
	// FM
	private var _MUL_TABLE :Vector.<int> = null;
	// LFO
	private var   _LFO_WAVE_TABLES      :Vector.<Vector.<int>> = null;
	private const _LFO_TIMER_INIT_VALUE :int = SynthData.LFO_TIMER_INIT_VALUE;
	private const _LFO_WAVE_TABLE_MASK  :int = SynthData.LFO_WAVE_TABLE_SIZE - 1;
	// Synth
	internal var VOICE_PARTS     :Vector.<VoicePart> = null;
	internal var WAVE_TABLES     :Vector.<Vector.<int>> = null;
	internal var TONE_PRESETS    :Vector.<TonePresetFM4> = null;
	internal var SAMPLE_DATAS    :Vector.<SampleData> = null;
	internal var SAMPLER_PRESETS :Vector.<SamplerPreset> = null;
	internal var pgSamplePow2tIdx :int = 0;  // PG PhaseStep (Pow2Table Index Offset) - SamplingRate
	internal var pgKeyPow2tIdx    :int = 0;  // PG PhaseStep (Pow2Table Index Offset) - ReferenceKey (NoteNumber 0)
	internal var egPow2tIdx       :int = 0;  // EG PhaseStep (Pow2Table Index Offset)
	internal var egPhaseStepSndOff:int = 0;  // EG PhaseStep - SoundOff
	internal var lfoTimerCoef     :int = 0;  // Coefficient Value - LFO Phase Update Timer
	// Pitch
	private var _ptBend     :int = 0;   // pitchBend
	private var _ptBendSens :int = 0;   // pitchBend Sensitivity
	// LFO
	private var _lfoPhase     :int = 0;  // LFO Phase
	private var _lfoTimer     :int = 0;  // LFO Phase Update Timer
	private var _lfoTimerStep :int = 0;  // Update Timer Step
// Operator 0,1 ---
	private var _lfoWaveTable :Vector.<int> = null;
	private var _pgBend       :int = 0;  // v = _ptBend * _ptBendSens
	private var _fdbk         :int = 0;
	
// Operator 0 ---
	private var _lfoMod_0     :int = 0;  // Modulation Depth
	private var _lfoModSens_0 :int = 0;  // Modulation Sensitivity
	private var _lfoOutPM_0   :int = 0;  // output - phase modulation
	private var _lfoOutAM_0   :int = 0;  // output - amplitude modulation
	// Wave
	private var _waveTable_0  :Vector.<int> = null;
	// PG
	private var _pgKey_0      :int = 0;
	private var _pgPhase_0    :int = 0;
	private var _pgPhaseStep_0:int = 0;
	// EG
	/*
	- Envelop Generator State -
	[ 5674 3210 ] 8Bits Flag Status (0 = deActive)
	[ ---- RSDA ] "-" = Reserved Bits
		A = Attack  stage
		D = Decay   stage
		S = Sustain stage
		R = Release stage
	*/
	private var _egState_0      :int = 0;
	private var _egLevelTable_0 :Vector.<int> = null;
	private var _egCount_0      :int = 0;
	private var _egCountStep_0  :int = 0;
	private var _egCountStepDR_0:int = 0;
	private var _egCountStepRR_0:int = 0;
	private var _egCntEnvSus_0  :int = 0;
	private var _egCntEnvBtm_0  :int = 0;
	private var _egGate_0       :int = 0;
	private var _egSL_0         :int = 0;
	private var _egTLv_0        :int = 0;
// Operator 1 ---
	private var _lfoMod_1     :int = 0;
	private var _lfoModSens_1 :int = 0;
	private var _lfoOutPM_1   :int = 0;
	private var _lfoOutAM_1   :int = 0;
	// Wave
	private var _waveTable_1  :Vector.<int> = null;
	// PG
	private var _pgKey_1      :int = 0;
	private var _pgPhase_1    :int = 0;
	private var _pgPhaseStep_1:int = 0;
	// EG
	private var _egState_1      :int = 0;
	private var _egLevelTable_1 :Vector.<int> = null;
	private var _egCount_1      :int = 0;
	private var _egCountStep_1  :int = 0;
	private var _egCountStepDR_1:int = 0;
	private var _egCountStepRR_1:int = 0;
	private var _egCntEnvSus_1  :int = 0;
	private var _egCntEnvBtm_1  :int = 0;
	private var _egGate_1       :int = 0;
	private var _egSL_1         :int = 0;
	private var _egTLv_1        :int = 0;
// Operator 1 ---
	private var _lfoMod_2     :int = 0;
	private var _lfoModSens_2 :int = 0;
	private var _lfoOutPM_2   :int = 0;
	private var _lfoOutAM_2   :int = 0;
	// Wave
	private var _waveTable_2  :Vector.<int> = null;
	// PG
	private var _pgKey_2      :int = 0;
	private var _pgPhase_2    :int = 0;
	private var _pgPhaseStep_2:int = 0;
	// EG
	private var _egState_2      :int = 0;
	private var _egLevelTable_2 :Vector.<int> = null;
	private var _egCount_2      :int = 0;
	private var _egCountStep_2  :int = 0;
	private var _egCountStepDR_2:int = 0;
	private var _egCountStepRR_2:int = 0;
	private var _egCntEnvSus_2  :int = 0;
	private var _egCntEnvBtm_2  :int = 0;
	private var _egGate_2       :int = 0;
	private var _egSL_2         :int = 0;
	private var _egTLv_2        :int = 0;
// Operator 1 ---
	private var _lfoMod_3     :int = 0;
	private var _lfoModSens_3 :int = 0;
	private var _lfoOutPM_3   :int = 0;
	private var _lfoOutAM_3   :int = 0;
	// Wave
	private var _waveTable_3  :Vector.<int> = null;
	// PG
	private var _pgKey_3      :int = 0;
	private var _pgPhase_3    :int = 0;
	private var _pgPhaseStep_3:int = 0;
	// EG
	private var _egState_3      :int = 0;
	private var _egLevelTable_3 :Vector.<int> = null;
	private var _egCount_3      :int = 0;
	private var _egCountStep_3  :int = 0;
	private var _egCountStepDR_3:int = 0;
	private var _egCountStepRR_3:int = 0;
	private var _egCntEnvSus_3  :int = 0;
	private var _egCntEnvBtm_3  :int = 0;
	private var _egGate_3       :int = 0;
	private var _egSL_3         :int = 0;
	private var _egTLv_3        :int = 0;
// Sampler
	private var _pgPhaseAFP :int = 0;  // PG Phase (After the Fixed Point)
	private var _plbkE      :int = 0;  // PlayBack End Frame   [plbkE <= channelPerSamples]
	private var _loopE      :int = 0;  // Loop End Frame       [loopE <= plbkE]
	private var _loopF      :int = 0;  // Loop Frames          [val = loopE - lpStart]
	private var _loopFk     :int = 0;  // Loop Frames          * after -> keyOff
	
	override internal function keyOff():void
	{
		// KeyOn -->
		if (state & 0x0010)
		{
		// OP0 is Active -->
			if (state & 1)
			{
				if (_egState_0 == 1)
				{
					_egLevelTable_0 = _EG_LEVEL_TABLE_DW;
					if (_egCount_0 >= _EG_MAX_CNT)_egCount_0 = 0;
					else _egCount_0 = _EG_LEVEL_TABLE_UP[_egCount_0 >> _EG_CNT_FP] << _EG_CNT_FP - 1;  // -1 = 8192 to 4096
				}
				_egGate_0 = _egCntEnvBtm_0;
				_egCountStep_0 = _egCountStepRR_0;
				_egState_0 = 8;
				if (_egCount_0 > _egCntEnvBtm_0)
				{ 
					if (state & 0xe){ state &= 0xfe; _egState_0 = 0; }
					else { killProcess(); return; }
				}
				// sampler ---
				_loopE = _plbkE;
				_loopF = _loopFk;
			}
		// OP1 is Active -->
			if (state & 2)
			{
				if (_egState_1 == 1)
				{
					_egLevelTable_1 = _EG_LEVEL_TABLE_DW;
					if (_egCount_1 >= _EG_MAX_CNT)_egCount_1 = 0;
					else _egCount_1 = _EG_LEVEL_TABLE_UP[_egCount_1 >> _EG_CNT_FP] << _EG_CNT_FP - 1;  // -1 = 8192 to 4096
				}
				_egGate_1 = _egCntEnvBtm_1;
				_egCountStep_1 = _egCountStepRR_1;
				_egState_1 = 8;
				if (_egCount_1 > _egCntEnvBtm_1)
				{ 
					if (state & 0xd){ state &= 0xfd; _egState_1 = 0; }
					else { killProcess(); return; }
				}
			}
			// OP2 is Active -->
			if (state & 4)
			{
				if (_egState_2 == 1)
				{
					_egLevelTable_2 = _EG_LEVEL_TABLE_DW;
					if (_egCount_2 >= _EG_MAX_CNT)_egCount_2 = 0;
					else _egCount_2 = _EG_LEVEL_TABLE_UP[_egCount_2 >> _EG_CNT_FP] << _EG_CNT_FP - 1;  // -1 = 8192 to 4096
				}
				_egGate_2 = _egCntEnvBtm_2;
				_egCountStep_2 = _egCountStepRR_2;
				_egState_2 = 8;
				if (_egCount_2 > _egCntEnvBtm_2)
				{
					if (state & 0xb) { state &= 0xfb; _egState_2 = 0; }
					else { killProcess(); return; }
				}
			}
			// OP3 is Active -->
			if (state & 8)
			{
				if (_egState_3 == 1)
				{
					_egLevelTable_3 = _EG_LEVEL_TABLE_DW;
					if (_egCount_3 >= _EG_MAX_CNT)_egCount_3 = 0;
					else _egCount_3 = _EG_LEVEL_TABLE_UP[_egCount_3 >> _EG_CNT_FP] << _EG_CNT_FP - 1;  // -1 = 8192 to 4096
				}
				_egGate_3 = _egCntEnvBtm_3;
				_egCountStep_3 = _egCountStepRR_3;
				_egState_3 = 8;
				if (_egCount_3 > _egCntEnvBtm_3)
				{
					if (state & 0x7) { state &= 0xf7; _egState_3 = 0; }
					else { killProcess(); return; }
				}
			}
			state = 0x0020 | state & 0xf;
		}
	}
	override internal function soundOff():void
	{
		// KeyOn || KeyOff --->
		if (state & 0x0030)
		{
		// OP0 Active --->
			if (state & 1)
			{
				if (_egState_0 == 1)
				{
					_egLevelTable_0 = _EG_LEVEL_TABLE_DW;
					if (_egCount_0 >= _EG_MAX_CNT)_egCount_0 = 0;
					else _egCount_0 = _EG_LEVEL_TABLE_UP[_egCount_0 >> _EG_CNT_FP] << _EG_CNT_FP - 1;  // -1 = 8192 to 4096
				}
				if (_egCount_0 > _egCntEnvBtm_0)
				{ 
					if (state & 0xe){ state &= 0xfe; }
					else { killProcess(); return; }
				}
				_egGate_0 = _egCntEnvBtm_0;
				_egCountStep_0 = egPhaseStepSndOff;
				_egState_0 = 0;
				// sampler ---
				_loopE = _plbkE;
				_loopF = _loopFk;
			}
		// OP1 Active --->
			if (state & 2)
			{
				if (_egState_1 == 1)
				{
					_egLevelTable_1 = _EG_LEVEL_TABLE_DW;
					if (_egCount_1 >= _EG_MAX_CNT)_egCount_1 = 0;
					else _egCount_1 = _EG_LEVEL_TABLE_UP[_egCount_1 >> _EG_CNT_FP] << _EG_CNT_FP - 1;  // -1 = 8192 to 4096
				}
				if (_egCount_1 > _egCntEnvBtm_1)
				{ 
					if (state & 0xd){ state &= 0xfd; }
					else { killProcess(); return; }
				}
				_egGate_1 = _egCntEnvBtm_1;
				_egCountStep_1 = egPhaseStepSndOff;
				_egState_1 = 0;
			}
		// OP2 Active --->
			if (state & 4)
			{
				if (_egState_2 == 1)
				{
					_egLevelTable_2 = _EG_LEVEL_TABLE_DW;
					if (_egCount_2 >= _EG_MAX_CNT)_egCount_2 = 0;
					else _egCount_2 = _EG_LEVEL_TABLE_UP[_egCount_2 >> _EG_CNT_FP] << _EG_CNT_FP - 1;  // -1 = 8192 to 4096
				}
				if (_egCount_2 > _egCntEnvBtm_2)
				{ 
					if (state & 0xb){ state &= 0xfb; }
					else { killProcess(); return; }
				}
				_egGate_2 = _egCntEnvBtm_2;
				_egCountStep_2 = egPhaseStepSndOff;
				_egState_2 = 0;
				// sampler ---
				_loopE = _plbkE;
				_loopF = _loopFk;
			}
		// OP3 Active --->
			if (state & 8)
			{
				if (_egState_3 == 1)
				{
					_egLevelTable_3 = _EG_LEVEL_TABLE_DW;
					if (_egCount_3 >= _EG_MAX_CNT)_egCount_3 = 0;
					else _egCount_3 = _EG_LEVEL_TABLE_UP[_egCount_3 >> _EG_CNT_FP] << _EG_CNT_FP - 1;  // -1 = 8192 to 4096
				}
				if (_egCount_3 > _egCntEnvBtm_3)
				{
					if (state & 0x7) { state &= 0xf7; }
					else { killProcess(); return; }
				}
				_egGate_3 = _egCntEnvBtm_3;
				_egCountStep_3 = egPhaseStepSndOff;
				_egState_3 = 0;
				// sampler ---
				_loopE = _plbkE;
				_loopF = _loopFk;
			}
			state = 0x0040 | state & 0xf;
			partNumber = -1; // SoundOff
		}
	}
	private function _shiftEGState_0():int
	{
		switch (_egState_0)
		{
			// ATTACK ->
			case 1:
				_egLevelTable_0 = _EG_LEVEL_TABLE_DW;
				_egCount_0 = 0;
				// -> DECAY
				if (_egSL_0 > 0)
				{
					_egGate_0 = _egCntEnvSus_0;
					_egCountStep_0 = _egCountStepDR_0;
					return _egState_0 = 2;
				}
				// -> SUSTAIN
				_egGate_0 = 0x7fffffff;
				_egCountStep_0 = 0;
				return _egState_0 = 4;
			// DECAY ->
			case 2:
				// -> SUSTAIN
				if (_egCntEnvSus_0 < _egCntEnvBtm_0)
				{
					_egLevelTable_0 = _EG_LEVEL_TABLE_DW;
					_egGate_0 = 0x7fffffff;
					_egCountStep_0 = 0;
					_egCount_0 = _egCntEnvSus_0;
					return _egState_0 = 4;
				}
			// DECAY,RELEASE,SoundOFF -> EG OFF
			default: return _egState_0 = 0;
		}
	}
	private function _shiftEGState_1():int
	{
		switch (_egState_1)
		{
			// ATTACK ->
			case 1:
				_egLevelTable_1 = _EG_LEVEL_TABLE_DW;
				_egCount_1 = 0;
				// -> DECAY
				if (_egSL_1 > 0)
				{
					_egGate_1 =  _egCntEnvSus_1;
					_egCountStep_1 = _egCountStepDR_1;
					return _egState_1 = 2;
				}
				// -> SUSTAIN
				_egGate_1 = 0x7fffffff;
				_egCountStep_1 = 0;
				return _egState_1 = 4;
			// DECAY ->
			case 2:
				// -> SUSTAIN
				if (_egCntEnvSus_1 < _egCntEnvBtm_1)
				{
					_egLevelTable_1 = _EG_LEVEL_TABLE_DW;
					_egGate_1 = 0x7fffffff;
					_egCountStep_1 = 0;
					_egCount_1 = _egCntEnvSus_1;
					return _egState_1 = 4;
				}
			// DECAY,RELEASE,SoundOFF -> EG OFF
			default: return _egState_1 = 0;
		}
	}
	private function _shiftEGState_2():int
	{
		switch (_egState_2)
		{
			// ATTACK ->
			case 1:
				_egLevelTable_2 = _EG_LEVEL_TABLE_DW;
				_egCount_2 = 0;
				// -> DECAY
				if (_egSL_2 > 0)
				{
					_egGate_2 = _egCntEnvSus_2;
					_egCountStep_2 = _egCountStepDR_2;
					return _egState_2 = 2;
				}
				// -> SUSTAIN
				_egGate_2 = 0x7fffffff;
				_egCountStep_2 = 0;
				return _egState_2 = 4;
			// DECAY ->
			case 2:
				// -> SUSTAIN
				if (_egCntEnvSus_2 < _egCntEnvBtm_2)
				{
					_egLevelTable_2 = _EG_LEVEL_TABLE_DW;
					_egGate_2 = 0x7fffffff;
					_egCountStep_2 = 0;
					_egCount_2 = _egCntEnvSus_2;
					return _egState_2 = 4;
				}
			// DECAY,RELEASE,SoundOFF -> EG OFF
			default: return _egState_2 = 0;
		}
	}
	private function _shiftEGState_3():int
	{
		switch (_egState_3)
		{
			// ATTACK ->
			case 1:
				_egLevelTable_3 = _EG_LEVEL_TABLE_DW;
				_egCount_3 = 0;
				// -> DECAY
				if (_egSL_3 > 0)
				{
					_egGate_3 =  _egCntEnvSus_3;
					_egCountStep_3 = _egCountStepDR_3;
					return _egState_3 = 2;
				}
				// -> SUSTAIN
				_egGate_3 = 0x7fffffff;
				_egCountStep_3 = 0;
				return _egState_3 = 4;
			// DECAY ->
			case 2:
				// -> SUSTAIN
				if (_egCntEnvSus_3 < _egCntEnvBtm_3)
				{
					_egLevelTable_3 = _EG_LEVEL_TABLE_DW;
					_egGate_3 = 0x7fffffff;
					_egCountStep_3 = 0;
					_egCount_3 = _egCntEnvSus_3;
					return _egState_3 = 4;
				}
			// DECAY,RELEASE,SoundOFF -> EG OFF
			default: return _egState_3 = 0;
		}
	}
	override internal function keyOn(ch:int, note:int, velo:int):void
	{
	// KeyOn Status ---
		state = 0x1f;
		partNumber = ch;
		noteNumber = note;
	// tone ---
		var _part:VoicePart = VOICE_PARTS[partNumber], _preset:TonePresetFM4, 
		_keyFlw_0:int = egPow2tIdx, 
		_keyFlw_1:int = egPow2tIdx, 
		_keyFlw_2:int = egPow2tIdx, 
		_keyFlw_3:int = egPow2tIdx;
		if (_part.mode == 2)
		{
			_preset = TONE_PRESETS[noteNumber | 128];
			_pgKey_0 = _POW2T_KEY_TABLE[_preset.key] + _MUL_TABLE[_preset.mul_0] + (_preset.tune_0 - 8192) + pgKeyPow2tIdx;
			_pgKey_1 = _POW2T_KEY_TABLE[_preset.key] + _MUL_TABLE[_preset.mul_1] + (_preset.tune_1 - 8192) + pgKeyPow2tIdx;
			_pgKey_2 = _POW2T_KEY_TABLE[_preset.key] + _MUL_TABLE[_preset.mul_2] + (_preset.tune_2 - 8192) + pgKeyPow2tIdx;
			_pgKey_3 = _POW2T_KEY_TABLE[_preset.key] + _MUL_TABLE[_preset.mul_3] + (_preset.tune_3 - 8192) + pgKeyPow2tIdx;
			//_keyFlw = 0;
			_prPan = _preset.pan;
		}
		else
		{
			_preset = TONE_PRESETS[_part.programChange];
			_pgKey_0 = _POW2T_KEY_TABLE[noteNumber] + _MUL_TABLE[_preset.mul_0] + (_preset.tune_0 - 8192) + pgKeyPow2tIdx;
			_keyFlw_0 += _EG_KEY_TABLE[127 + noteNumber - _preset.kfStart_0] << _preset.kf_0;
			_pgKey_1 = _POW2T_KEY_TABLE[noteNumber] + _MUL_TABLE[_preset.mul_1] + (_preset.tune_1 - 8192) + pgKeyPow2tIdx;
			_keyFlw_1 += _EG_KEY_TABLE[127 + noteNumber - _preset.kfStart_1] << _preset.kf_1;
			_pgKey_2 = _POW2T_KEY_TABLE[noteNumber] + _MUL_TABLE[_preset.mul_2] + (_preset.tune_2 - 8192) + pgKeyPow2tIdx;
			_keyFlw_2 += _EG_KEY_TABLE[127 + noteNumber - _preset.kfStart_2] << _preset.kf_2;
			_pgKey_3 = _POW2T_KEY_TABLE[noteNumber] + _MUL_TABLE[_preset.mul_3] + (_preset.tune_3 - 8192) + pgKeyPow2tIdx;
			_keyFlw_3 += _EG_KEY_TABLE[127 + noteNumber - _preset.kfStart_3] << _preset.kf_3;
			_prPan = 64;
		}
		_lfoWaveTable = _LFO_WAVE_TABLES[_preset.lfoWave];
		_lfoTimerStep = _preset.lfoFreq * lfoTimerCoef;
		_lfoTimer = _LFO_TIMER_INIT_VALUE;
		_lfoPhase = _preset.strc >> 3;  // ~8 AM 
	// volume ---
		_vPtVol = _mainVolTable[_part.ccValues[7]];
		_vPtExp = _exprVolTable[_part.ccValues[11]];
		_vVel = _veloVolTable[velo];
		_vVol = _vVel * _masterVolume;
		_vOut = _vPtVol * _vPtExp * _vVol;
		_vPanL = _panTable[      _prPan + _part.ccValues[10]];
		_vPanR = _panTable[256 - _prPan - _part.ccValues[10]];
		_vOutL = _vOut * _vPanL;
		_vOutR = _vOut * _vPanR;
	// pitch ---
		_ptBendSens = _POW2T_KEY_TABLE[_part.bendSens];
		_ptBend = _part.pitchBend - 8192;
		_pgBend = _ptBend * _ptBendSens >> 13;
	// process ---
		process = _PROC_PTRS[_preset.strc];
		_fdbk = _EG_TOTAL_LEVEL_TABLE[_preset.fl];
// Operator 0 --->
		_waveTable_0 = WAVE_TABLES[_preset.wave_0];
	// LFO ---
		_lfoModSens_0 = _preset.lms_0;
		_lfoMod_0 = _part.ccValues[1] * _lfoModSens_0;
		_lfoOutPM_0 = 0;
		_lfoOutAM_0 = 0;
	// PG ---
		var v:int = _pgKey_0 + _pgBend;
		_pgPhaseStep_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		_pgPhase_0 = 0;
	// EG ---
		v = _preset.tl_0 + (127 - velo >> 7 - _preset.tlVelCtrl_0);
		if (v > 127)v = 127;
		_egTLv_0 = _EG_TOTAL_LEVEL_TABLE[v];
		_egCntEnvBtm_0 = _EG_CNT_GATE_TABLE[128 - v];
		_egSL_0 = _preset.sl_0;
		_egCntEnvSus_0 = _EG_CNT_GATE_TABLE[_egSL_0];
		if (_egCntEnvSus_0 > _egCntEnvBtm_0)_egCntEnvSus_0 = _egCntEnvBtm_0;
		v = (_preset.dr_0 << _EG_PRAM_OCTV_LS) + _keyFlw_0;
		_egCountStepDR_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		v = (_preset.rr_0 << _EG_PRAM_OCTV_LS) + _keyFlw_0;
		_egCountStepRR_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
	// EG State ---
		// -> ATTACK
		if (_preset.ar_0 < 127) {
			_egCount_0 = _EG_CNT_ENV_ATK;
			_egLevelTable_0 = _EG_LEVEL_TABLE_UP;
			_egGate_0 = _EG_MAX_CNT;
			v = (_preset.ar_0 << _EG_PRAM_OCTV_LS) + _keyFlw_0;
			_egCountStep_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
			_egState_0 = 1;
		} else {
			_egLevelTable_0 = _EG_LEVEL_TABLE_DW;
			_egCount_0 = 0;
		// -> DECAY
			if (_egSL_0 > 0) {
				_egGate_0 = _egCntEnvSus_0;
				_egCountStep_0 = _egCountStepDR_0;
				_egState_0 = 2;
		// -> SUSTAIN
			} else {
				_egGate_0 = 0x7fffffff;
				_egCountStep_0 = 0;
				_egState_0 = 4; }
		}
// Operator 1 --->
		_waveTable_1 = WAVE_TABLES[_preset.wave_1];
	// LFO ---
		_lfoModSens_1 = _preset.lms_1;
		_lfoMod_1 = _part.ccValues[1] * _lfoModSens_1;
		_lfoOutPM_1 = 0;
		_lfoOutAM_1 = 0;
	// PG ---
		v = _pgKey_1 + _pgBend;
		_pgPhaseStep_1 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		_pgPhase_1 = 0;
	// EG ---
		v = _preset.tl_1 + (127 - velo >> 7 - _preset.tlVelCtrl_1);
		if (v > 127)v = 127;
		_egTLv_1 = _EG_TOTAL_LEVEL_TABLE[v];
		_egCntEnvBtm_1 = _EG_CNT_GATE_TABLE[128 - v];
		_egSL_1 = _preset.sl_1;
		_egCntEnvSus_1 = _EG_CNT_GATE_TABLE[_egSL_1];
		if (_egCntEnvSus_1 > _egCntEnvBtm_1)_egCntEnvSus_1 = _egCntEnvBtm_1;
		v = (_preset.dr_1 << _EG_PRAM_OCTV_LS) + _keyFlw_1;
		_egCountStepDR_1 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		v = (_preset.rr_1 << _EG_PRAM_OCTV_LS) + _keyFlw_1;
		_egCountStepRR_1 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
	// EG State ---
		// -> ATTACK
		if (_preset.ar_1 < 127) {
			_egCount_1 = _EG_CNT_ENV_ATK;
			_egLevelTable_1 = _EG_LEVEL_TABLE_UP;
			_egGate_1 = _EG_MAX_CNT;
			v = (_preset.ar_1 << _EG_PRAM_OCTV_LS) + _keyFlw_1;
			_egCountStep_1 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
			_egState_1 = 1;
		} else {
			_egLevelTable_1 = _EG_LEVEL_TABLE_DW;
			_egCount_1 = 0;
		// -> DECAY
			if (_egSL_1 > 0) {
				_egGate_1 = _egCntEnvSus_1;
				_egCountStep_1 = _egCountStepDR_1;
				_egState_1 = 2;
		// -> SUSTAIN
			} else {
				_egGate_1 = 0x7fffffff;
				_egCountStep_1 = 0;
				_egState_1 = 4; }
		}
// Operator 2 --->
		_waveTable_2 = WAVE_TABLES[_preset.wave_2];
	// LFO ---
		_lfoModSens_2 = _preset.lms_2;
		_lfoMod_2 = _part.ccValues[1] * _lfoModSens_2;
		_lfoOutPM_2 = 0;
		_lfoOutAM_2 = 0;
	// PG ---
		v = _pgKey_2 + _pgBend;
		_pgPhaseStep_2 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		_pgPhase_2 = 0;
	// EG ---
		v = _preset.tl_2 + (127 - velo >> 7 - _preset.tlVelCtrl_2);
		if (v > 127)v = 127;
		_egTLv_2 = _EG_TOTAL_LEVEL_TABLE[v];
		_egCntEnvBtm_2 = _EG_CNT_GATE_TABLE[128 - v];
		_egSL_2 = _preset.sl_2;
		_egCntEnvSus_2 = _EG_CNT_GATE_TABLE[_egSL_2];
		if (_egCntEnvSus_2 > _egCntEnvBtm_2)_egCntEnvSus_2 = _egCntEnvBtm_2;
		v = (_preset.dr_2 << _EG_PRAM_OCTV_LS) + _keyFlw_2;
		_egCountStepDR_2 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		v = (_preset.rr_2 << _EG_PRAM_OCTV_LS) + _keyFlw_2;
		_egCountStepRR_2 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
	// EG State ---
		// -> ATTACK
		if (_preset.ar_2 < 127) {
			_egCount_2 = _EG_CNT_ENV_ATK;
			_egLevelTable_2 = _EG_LEVEL_TABLE_UP;
			_egGate_2 = _EG_MAX_CNT;
			v = (_preset.ar_2 << _EG_PRAM_OCTV_LS) + _keyFlw_2;
			_egCountStep_2 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
			_egState_2 = 1;
		} else {
			_egLevelTable_2 = _EG_LEVEL_TABLE_DW;
			_egCount_2 = 0;
		// -> DECAY
			if (_egSL_2 > 0) {
				_egGate_2 = _egCntEnvSus_2;
				_egCountStep_2 = _egCountStepDR_2;
				_egState_2 = 2;
		// -> SUSTAIN
			} else {
				_egGate_2 = 0x7fffffff;
				_egCountStep_2 = 0;
				_egState_2 = 4; }
		}
// Operator 3 --->
		_waveTable_3 = WAVE_TABLES[_preset.wave_3];
	// LFO ---
		_lfoModSens_3 = _preset.lms_3;
		_lfoMod_3 = _part.ccValues[1] * _lfoModSens_3;
		_lfoOutPM_3 = 0;
		_lfoOutAM_3 = 0;
	// PG ---
		v = _pgKey_3 + _pgBend;
		_pgPhaseStep_3 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		_pgPhase_3 = 0;
	// EG ---
		v = _preset.tl_3 + (127 - velo >> 7 - _preset.tlVelCtrl_3);
		if (v > 127)v = 127;
		_egTLv_3 = _EG_TOTAL_LEVEL_TABLE[v];
		_egCntEnvBtm_3 = _EG_CNT_GATE_TABLE[128 - v];
		_egSL_3 = _preset.sl_3;
		_egCntEnvSus_3 = _EG_CNT_GATE_TABLE[_egSL_3];
		if (_egCntEnvSus_3 > _egCntEnvBtm_3)_egCntEnvSus_3 = _egCntEnvBtm_3;
		v = (_preset.dr_3 << _EG_PRAM_OCTV_LS) + _keyFlw_3;
		_egCountStepDR_3 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		v = (_preset.rr_3 << _EG_PRAM_OCTV_LS) + _keyFlw_3;
		_egCountStepRR_3 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
	// EG State ---
		// -> ATTACK
		if (_preset.ar_3 < 127) {
			_egCount_3 = _EG_CNT_ENV_ATK;
			_egLevelTable_3 = _EG_LEVEL_TABLE_UP;
			_egGate_3 = _EG_MAX_CNT;
			v = (_preset.ar_3 << _EG_PRAM_OCTV_LS) + _keyFlw_3;
			_egCountStep_3 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
			_egState_3 = 1;
		} else {
			_egLevelTable_3 = _EG_LEVEL_TABLE_DW;
			_egCount_3 = 0;
		// -> DECAY
			if (_egSL_3 > 0) {
				_egGate_3 = _egCntEnvSus_3;
				_egCountStep_3 = _egCountStepDR_3;
				_egState_3 = 2;
		// -> SUSTAIN
			} else {
				_egGate_3 = 0x7fffffff;
				_egCountStep_3 = 0;
				_egState_3 = 4; }
		}
	}
	internal function keyOnSampler(ch:int, note:int, velo:int):void
	{
	// KeyOn Status ---
		state = 0x11;
		partNumber = ch;
		noteNumber = note;
	// tone ---
		var _part:VoicePart = VOICE_PARTS[partNumber],
			_preset:SamplerPreset = SAMPLER_PRESETS[noteNumber],
			_sample:SampleData = SAMPLE_DATAS[_preset.smplNum];
		if (_sample.sampleData == null){ killProcess(); return; }
		_waveTable_0 = _sample.sampleData;
		_plbkE = _sample.plbkE;
		_loopE = _sample.loopE;
		_loopF = _sample.loopF;
		_loopFk = _sample.loopFk;
	// LFO ---
		_lfoOutPM_0 = 0;
		_lfoOutAM_0 = 0;
	// volume ---
		_vPtVol = _mainVolTable[_part.ccValues[7]];
		_vPtExp = _exprVolTable[_part.ccValues[11]];
		_vVel = _veloVolTable[velo];
		_vVol = _vVel * _masterVolume;
		_vOut = _vPtVol * _vPtExp * _vVol;
		_prPan = _preset.pan;
		_vPanL = _panTable[      _prPan + _part.ccValues[10]];
		_vPanR = _panTable[256 - _prPan - _part.ccValues[10]];
		_vOutL = _vOut * _vPanL;
		_vOutR = _vOut * _vPanR;
	// PG ---
		_ptBendSens = _POW2T_KEY_TABLE[_part.bendSens];
		_ptBend = _part.pitchBend - 8192;
		_pgBend = _ptBend * _ptBendSens >> 13;
		_pgKey_0 = (_preset.octv - 8 << 13) + (_preset.tune - 8192) + _sample.pgSample + pgSamplePow2tIdx;
		var v:int = _pgKey_0 + _pgBend;
		_pgPhaseStep_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		_pgPhase_0 = _sample.plbkS;
		_pgPhaseAFP = 0;
	// EG ---
		_egTLv_0 = _EG_TOTAL_LEVEL_TABLE[_preset.tl];
		_egCntEnvBtm_0 = _EG_CNT_GATE_TABLE[128 - _preset.tl];
		_egSL_0 = _preset.sl;
		_egCntEnvSus_0 = _EG_CNT_GATE_TABLE[_egSL_0];
		if (_egCntEnvSus_0 > _egCntEnvBtm_0)_egCntEnvSus_0 = _egCntEnvBtm_0;
		v = (_preset.dr << _EG_PRAM_OCTV_LS) + egPow2tIdx;
		_egCountStepDR_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		v = (_preset.rr << _EG_PRAM_OCTV_LS) + egPow2tIdx;
		_egCountStepRR_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
	// EG State ---
		// -> ATTACK
		if (_preset.ar < 127) {
			_egCount_0 = _EG_CNT_ENV_ATK;
			_egLevelTable_0 = _EG_LEVEL_TABLE_UP;
			_egGate_0 = _EG_MAX_CNT;
			v = (_preset.ar << _EG_PRAM_OCTV_LS) + egPow2tIdx;
			_egCountStep_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _EG_POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
			_egState_0 = 1;
		} else {
			_egLevelTable_0 = _EG_LEVEL_TABLE_DW;
			_egCount_0 = 0;
		// -> DECAY
			if (_egSL_0 > 0) {
				_egGate_0 = _egCntEnvSus_0;
				_egCountStep_0 = _egCountStepDR_0;
				_egState_0 = 2;
		// -> SUSTAIN
			} else {
				_egGate_0 = 0x7fffffff;
				_egCountStep_0 = 0;
				_egState_0 = 4; }
		}
	// process ---
		process = (_sample.channelCount == 2)? _procSmpl2ch: _procSmpl1ch;
	}
	override internal function updPartBendSens(val:int):void
	{
		_ptBendSens = _POW2T_KEY_TABLE[val];
		_pgBend = _ptBend * _ptBendSens >> 13;
		var v:int;
		// Inst op0 --->
		if (state & 1)
		{
			v = _pgKey_0 + _pgBend + _lfoOutPM_0;
			_pgPhaseStep_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		}
		// Inst op1 --->
		if (state & 2)
		{
			v = _pgKey_1 + _pgBend + _lfoOutPM_1;
			_pgPhaseStep_1 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		}
		// Inst op2 --->
		if (state & 4)
		{
			v = _pgKey_2 + _pgBend + _lfoOutPM_2;
			_pgPhaseStep_2 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		}
		// Inst op3 --->
		if (state & 8)
		{
			v = _pgKey_3 + _pgBend + _lfoOutPM_3;
			_pgPhaseStep_3 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		}
	}
	override internal function updPartPitchBend(val:int):void
	{
		_ptBend = val - 8192;
		_pgBend = _ptBend * _ptBendSens >> 13;
		var v:int;
		// Inst op0 --->
		if (state & 1)
		{
			v = _pgKey_0 + _pgBend + _lfoOutPM_0;
			_pgPhaseStep_0 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		}
		// Inst op1 --->
		if (state & 2)
		{
			v = _pgKey_1 + _pgBend + _lfoOutPM_1;
			_pgPhaseStep_1 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		}
		// Inst op2 --->
		if (state & 4)
		{
			v = _pgKey_2 + _pgBend + _lfoOutPM_2;
			_pgPhaseStep_2 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		}
		// Inst op3 --->
		if (state & 8)
		{
			v = _pgKey_3 + _pgBend + _lfoOutPM_3;
			_pgPhaseStep_3 = _POW2_TABLE[v & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[v >> _POW2T_OCTV_BITS];
		}
	}
	override internal function updPartModulation(val:int):void
	{
		_lfoMod_0 = val * _lfoModSens_0;
		_lfoMod_1 = val * _lfoModSens_1;
		_lfoMod_2 = val * _lfoModSens_2;
		_lfoMod_3 = val * _lfoModSens_3;
	}
	internal function updateLfoFrequency(val:int):void
	{
		_lfoTimerStep = val * lfoTimerCoef;
	}
// setup
	internal function setup(_synthData:SynthData):void
	{
		// Volume Table
		_mainVolTable = _synthData.getVOLUME_TABLE_0dB();
		_exprVolTable = _synthData.getVOLUME_TABLE_0dB();
		_veloVolTable = _synthData.getVOLUME_TABLE_0dB();
		_panTable     = _synthData.getPAN_TABLE_SIN();
		// Log2 Volume Table
		_LOG2V_TABLE    = _synthData.getLOG2V_TABLE();
		_LOG2V_TABLE_I  = _synthData.getLOG2V_TABLE_I();
		// Pow2 Table
		_POW2_TABLE      = _synthData.getPOW2_TABLE();
		_POW2T_RS_TABLE  = _synthData.getPOW2T_RS_TABLE();
		_POW2T_KEY_TABLE = _synthData.getPOW2T_KEY_TABLE();
		// EG
		_EG_TOTAL_LEVEL_TABLE = _synthData.getEG_TOTAL_LEVEL_TABLE();
		_EG_LEVEL_TABLE_UP    = _synthData.getEG_LEVEL_TABLE_UP();
		_EG_LEVEL_TABLE_DW    = _synthData.getEG_LEVEL_TABLE_DW();
		_EG_KEY_TABLE         = _synthData.getEG_KEY_TABLE();
		_EG_POW2T_RS_TABLE    = _synthData.getEG_POW2T_RS_TABLE();
		_EG_CNT_GATE_TABLE    = _synthData.getEG_CNT_GATE_TABLE();
		// etc
		_MUL_TABLE       = _synthData.getMUL_TABLE();
		_LFO_WAVE_TABLES = _synthData.getLFO_WAVE_TABLES();
	}
// Constructor
	public function VoiceUnitFM4():void
	{
		super();
		_PROC_PTRS[0] = _proc0;
		_PROC_PTRS[1] = _proc1;
		_PROC_PTRS[2] = _proc2;
		_PROC_PTRS[3] = _proc3;
		_PROC_PTRS[4] = _proc4;
		_PROC_PTRS[5] = _proc5;
		_PROC_PTRS[6] = _proc6;
		_PROC_PTRS[7] = _proc7;
		_PROC_PTRS[8] = _proc8;
		_PROC_PTRS[9] = _proc9;
		_PROC_PTRS[10] = _procA;
		_PROC_PTRS[11] = _procB;
		_PROC_PTRS[12] = _procC;
		_PROC_PTRS[13] = _procD;
		_PROC_PTRS[14] = _procE;
		_PROC_PTRS[15] = _procF;
	}
// process ---
	// pointer array (process Function)
	private const _PROC_PTRS :Vector.<Function> = new Vector.<Function>(16, true);
	private function _proc0(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				// op0
				_lfoOutPM_0 = (p * _lfoMod_0) >> 11;  // 7(mod) + 4(sens)
				x = _pgKey_0 + _pgBend + _lfoOutPM_0;
				_pgPhaseStep_0 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_1 = (p * _lfoMod_1) >> 11;
				x = _pgKey_1 + _pgBend + _lfoOutPM_1;
				_pgPhaseStep_1 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op0
				_lfoOutPM_2 = (p * _lfoMod_2) >> 11;
				x = _pgKey_2 + _pgBend + _lfoOutPM_2;
				_pgPhaseStep_2 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_3 = (p * _lfoMod_3) >> 11;
				x = _pgKey_3 + _pgBend + _lfoOutPM_3;
				_pgPhaseStep_3 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// counter
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe){ state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				p = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				// OP 0 out --->
				n = _log2vT[p];
				// counter --->
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			} else n = 0.0;
		// Operator 1 --->
			if (state & 2)
			{
				// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1;
				// pg update --->
				x = _wt_1[_pgPhase_1 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2;
				// pg update --->
				x = _wt_2[_pgPhase_2 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else
					{
						if (state & 0x7) { state &= 0xf7; _egCount_3 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3;
				// pg update --->
				x = _wt_3[_pgPhase_3 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _proc1(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				// op0
				_lfoOutPM_0 = (p * _lfoMod_0) >> 11;  // 7(mod) + 4(sens)
				x = _pgKey_0 + _pgBend + _lfoOutPM_0;
				_pgPhaseStep_0 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_1 = (p * _lfoMod_1) >> 11;
				x = _pgKey_1 + _pgBend + _lfoOutPM_1;
				_pgPhaseStep_1 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op0
				_lfoOutPM_2 = (p * _lfoMod_2) >> 11;
				x = _pgKey_2 + _pgBend + _lfoOutPM_2;
				_pgPhaseStep_2 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_3 = (p * _lfoMod_3) >> 11;
				x = _pgKey_3 + _pgBend + _lfoOutPM_3;
				_pgPhaseStep_3 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// counter
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				p = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[p];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
			// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1;
			// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_1[p] + x;
			// OP 1 out ----->
				n = _log2vT[x];
			// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else n = 0.0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2;
				// pg update --->
				x = _wt_2[_pgPhase_2 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else
					{
						if (state & 0x7) { state &= 0xf7; _egCount_3 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3;
				// pg update --->
				x = _wt_3[_pgPhase_3 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _proc2(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				// op0
				_lfoOutPM_0 = (p * _lfoMod_0) >> 11;  // 7(mod) + 4(sens)
				x = _pgKey_0 + _pgBend + _lfoOutPM_0;
				_pgPhaseStep_0 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_1 = (p * _lfoMod_1) >> 11;
				x = _pgKey_1 + _pgBend + _lfoOutPM_1;
				_pgPhaseStep_1 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op0
				_lfoOutPM_2 = (p * _lfoMod_2) >> 11;
				x = _pgKey_2 + _pgBend + _lfoOutPM_2;
				_pgPhaseStep_2 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_3 = (p * _lfoMod_3) >> 11;
				x = _pgKey_3 + _pgBend + _lfoOutPM_3;
				_pgPhaseStep_3 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// counter
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				p = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[p];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
				// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1;
				// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_1[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else n = 0.0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2;
				// pg update --->
				p = (o + (_pgPhase_2 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_2[p] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else
					{
						if (state & 0x7) { state &= 0xf7; _egCount_3 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _proc3(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				// op0
				_lfoOutPM_0 = (p * _lfoMod_0) >> 11;  // 7(mod) + 4(sens)
				x = _pgKey_0 + _pgBend + _lfoOutPM_0;
				_pgPhaseStep_0 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_1 = (p * _lfoMod_1) >> 11;
				x = _pgKey_1 + _pgBend + _lfoOutPM_1;
				_pgPhaseStep_1 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op0
				_lfoOutPM_2 = (p * _lfoMod_2) >> 11;
				x = _pgKey_2 + _pgBend + _lfoOutPM_2;
				_pgPhaseStep_2 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_3 = (p * _lfoMod_3) >> 11;
				x = _pgKey_3 + _pgBend + _lfoOutPM_3;
				_pgPhaseStep_3 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// counter
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				x = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[x];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
				// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1;
				// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_1[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else n = 0.0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2;
				// pg update --->
				x = _wt_2[_pgPhase_2 >> _PG_PHASE_FP] + x;
				// OP 0 out --->
				o = _log2vTI[x];
				// counter --->	
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else
					{
						if (state & 0x7) { state &= 0xf7; _egCount_3 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _proc4(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				// op0
				_lfoOutPM_0 = (p * _lfoMod_0) >> 11;  // 7(mod) + 4(sens)
				x = _pgKey_0 + _pgBend + _lfoOutPM_0;
				_pgPhaseStep_0 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_1 = (p * _lfoMod_1) >> 11;
				x = _pgKey_1 + _pgBend + _lfoOutPM_1;
				_pgPhaseStep_1 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op0
				_lfoOutPM_2 = (p * _lfoMod_2) >> 11;
				x = _pgKey_2 + _pgBend + _lfoOutPM_2;
				_pgPhaseStep_2 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_3 = (p * _lfoMod_3) >> 11;
				x = _pgKey_3 + _pgBend + _lfoOutPM_3;
				_pgPhaseStep_3 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// counter
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				x = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[x];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
			// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1;
			// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_1[p] + x;
			// OP 1 out ----->
				o = _log2vTI[x];
			// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2;
				// pg update --->
				x = _wt_2[_pgPhase_2 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				o += _log2vTI[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else { killProcess(); return; }
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _proc5(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				// op0
				_lfoOutPM_0 = (p * _lfoMod_0) >> 11;  // 7(mod) + 4(sens)
				x = _pgKey_0 + _pgBend + _lfoOutPM_0;
				_pgPhaseStep_0 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_1 = (p * _lfoMod_1) >> 11;
				x = _pgKey_1 + _pgBend + _lfoOutPM_1;
				_pgPhaseStep_1 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op0
				_lfoOutPM_2 = (p * _lfoMod_2) >> 11;
				x = _pgKey_2 + _pgBend + _lfoOutPM_2;
				_pgPhaseStep_2 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_3 = (p * _lfoMod_3) >> 11;
				x = _pgKey_3 + _pgBend + _lfoOutPM_3;
				_pgPhaseStep_3 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// counter
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 1 --->
			if (state & 2)
			{
				// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1;
				// pg update --->
				p = _wt_1[_pgPhase_1 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				o = _log2vTI[p];
				// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2;
				// pg update --->
				p = (o + (_pgPhase_2 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_2[p] + x;
				// OP 1 out ----->
				o = _log2vTI[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				x = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o += _log2vTI[x];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else { killProcess(); return; }
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _proc6(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				// op0
				_lfoOutPM_0 = (p * _lfoMod_0) >> 11;  // 7(mod) + 4(sens)
				x = _pgKey_0 + _pgBend + _lfoOutPM_0;
				_pgPhaseStep_0 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_1 = (p * _lfoMod_1) >> 11;
				x = _pgKey_1 + _pgBend + _lfoOutPM_1;
				_pgPhaseStep_1 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op0
				_lfoOutPM_2 = (p * _lfoMod_2) >> 11;
				x = _pgKey_2 + _pgBend + _lfoOutPM_2;
				_pgPhaseStep_2 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_3 = (p * _lfoMod_3) >> 11;
				x = _pgKey_3 + _pgBend + _lfoOutPM_3;
				_pgPhaseStep_3 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// counter
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				x = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[x];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
			// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1;
			// pg update --->
				x = _wt_1[_pgPhase_1 >> _PG_PHASE_FP] + x;
			// OP 1 out ----->
				o += _log2vTI[x];
			// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2;
				// pg update --->
				p = (o + (_pgPhase_2 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_2[p] + x;
				// OP 1 out ----->
				o = _log2vTI[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else { killProcess(); return; }
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _proc7(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				// op0
				_lfoOutPM_0 = (p * _lfoMod_0) >> 11;  // 7(mod) + 4(sens)
				x = _pgKey_0 + _pgBend + _lfoOutPM_0;
				_pgPhaseStep_0 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_1 = (p * _lfoMod_1) >> 11;
				x = _pgKey_1 + _pgBend + _lfoOutPM_1;
				_pgPhaseStep_1 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op0
				_lfoOutPM_2 = (p * _lfoMod_2) >> 11;
				x = _pgKey_2 + _pgBend + _lfoOutPM_2;
				_pgPhaseStep_2 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// op1
				_lfoOutPM_3 = (p * _lfoMod_3) >> 11;
				x = _pgKey_3 + _pgBend + _lfoOutPM_3;
				_pgPhaseStep_3 = _POW2_TABLE[x & _POW2T_OCTV_MASK] >> _POW2T_RS_TABLE[x >> _POW2T_OCTV_BITS];
				// counter
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				p = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[p];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
			// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1;
			// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				p = _wt_1[p] + x;
			// OP 1 out ----->
				o = _log2vTI[p];
			// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2;
				// pg update --->
				p = (o + (_pgPhase_2 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				p = _wt_2[p] + x;
				// OP 1 out ----->
				o = _log2vTI[p];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else { killProcess(); return; }
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				p = _wt_3[p] + x;
				// OP 1 out ----->
				n = _log2vT[p];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}

	private function _proc8(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				x = (p * _lfoMod_0) >> 11;
				_lfoOutAM_0 = x & 0x7ffffffe;
				x = (p * _lfoMod_1) >> 11;
				_lfoOutAM_1 = x & 0x7ffffffe;
				x = (p * _lfoMod_2) >> 11;
				_lfoOutAM_2 = x & 0x7ffffffe;
				x = (p * _lfoMod_3) >> 11;
				_lfoOutAM_3 = x & 0x7ffffffe;
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe){ state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0 + _lfoOutAM_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				p = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				// OP 0 out --->
				n = _log2vT[p];
				// counter --->
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			} else n = 0.0;
		// Operator 1 --->
			if (state & 2)
			{
				// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1 + _lfoOutAM_1;
				// pg update --->
				x = _wt_1[_pgPhase_1 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2 + _lfoOutAM_2;
				// pg update --->
				x = _wt_2[_pgPhase_2 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else
					{
						if (state & 0x7) { state &= 0xf7; _egCount_3 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3 + _lfoOutAM_3;
				// pg update --->
				x = _wt_3[_pgPhase_3 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _proc9(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				x = (p * _lfoMod_0) >> 11;
				_lfoOutAM_0 = x & 0x7ffffffe;
				x = (p * _lfoMod_1) >> 11;
				_lfoOutAM_1 = x & 0x7ffffffe;
				x = (p * _lfoMod_2) >> 11;
				_lfoOutAM_2 = x & 0x7ffffffe;
				x = (p * _lfoMod_3) >> 11;
				_lfoOutAM_3 = x & 0x7ffffffe;
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0 + _lfoOutAM_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				p = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[p];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
			// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1 + _lfoOutAM_1;
			// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_1[p] + x;
			// OP 1 out ----->
				n = _log2vT[x];
			// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else n = 0.0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2 + _lfoOutAM_2;
				// pg update --->
				x = _wt_2[_pgPhase_2 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else
					{
						if (state & 0x7) { state &= 0xf7; _egCount_3 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3 + _lfoOutAM_3;
				// pg update --->
				x = _wt_3[_pgPhase_3 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _procA(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				x = (p * _lfoMod_0) >> 11;
				_lfoOutAM_0 = x & 0x7ffffffe;
				x = (p * _lfoMod_1) >> 11;
				_lfoOutAM_1 = x & 0x7ffffffe;
				x = (p * _lfoMod_2) >> 11;
				_lfoOutAM_2 = x & 0x7ffffffe;
				x = (p * _lfoMod_3) >> 11;
				_lfoOutAM_3 = x & 0x7ffffffe;
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0 + _lfoOutAM_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				p = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[p];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
				// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1 + _lfoOutAM_1;
				// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_1[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else n = 0.0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2 + _lfoOutAM_2;
				// pg update --->
				p = (o + (_pgPhase_2 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_2[p] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else
					{
						if (state & 0x7) { state &= 0xf7; _egCount_3 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3 + _lfoOutAM_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _procB(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				x = (p * _lfoMod_0) >> 11;
				_lfoOutAM_0 = x & 0x7ffffffe;
				x = (p * _lfoMod_1) >> 11;
				_lfoOutAM_1 = x & 0x7ffffffe;
				x = (p * _lfoMod_2) >> 11;
				_lfoOutAM_2 = x & 0x7ffffffe;
				x = (p * _lfoMod_3) >> 11;
				_lfoOutAM_3 = x & 0x7ffffffe;
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0 + _lfoOutAM_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				x = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[x];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
				// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1 + _lfoOutAM_1;
				// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_1[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else n = 0.0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2 + _lfoOutAM_2;
				// pg update --->
				x = _wt_2[_pgPhase_2 >> _PG_PHASE_FP] + x;
				// OP 0 out --->
				o = _log2vTI[x];
				// counter --->	
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else
					{
						if (state & 0x7) { state &= 0xf7; _egCount_3 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3 + _lfoOutAM_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n += _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _procC(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				x = (p * _lfoMod_0) >> 11;
				_lfoOutAM_0 = x & 0x7ffffffe;
				x = (p * _lfoMod_1) >> 11;
				_lfoOutAM_1 = x & 0x7ffffffe;
				x = (p * _lfoMod_2) >> 11;
				_lfoOutAM_2 = x & 0x7ffffffe;
				x = (p * _lfoMod_3) >> 11;
				_lfoOutAM_3 = x & 0x7ffffffe;
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0 + _lfoOutAM_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				x = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[x];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
			// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1 + _lfoOutAM_1;
			// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_1[p] + x;
			// OP 1 out ----->
				o = _log2vTI[x];
			// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2 + _lfoOutAM_2;
				// pg update --->
				x = _wt_2[_pgPhase_2 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				o += _log2vTI[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else { killProcess(); return; }
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3 + _lfoOutAM_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _procD(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				x = (p * _lfoMod_0) >> 11;
				_lfoOutAM_0 = x & 0x7ffffffe;
				x = (p * _lfoMod_1) >> 11;
				_lfoOutAM_1 = x & 0x7ffffffe;
				x = (p * _lfoMod_2) >> 11;
				_lfoOutAM_2 = x & 0x7ffffffe;
				x = (p * _lfoMod_3) >> 11;
				_lfoOutAM_3 = x & 0x7ffffffe;
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 1 --->
			if (state & 2)
			{
				// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1 + _lfoOutAM_1;
				// pg update --->
				p = _wt_1[_pgPhase_1 >> _PG_PHASE_FP] + x;
				// OP 1 out ----->
				o = _log2vTI[p];
				// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2 + _lfoOutAM_2;
				// pg update --->
				p = (o + (_pgPhase_2 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_2[p] + x;
				// OP 1 out ----->
				o = _log2vTI[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0 + _lfoOutAM_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				x = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o += _log2vTI[x];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else { killProcess(); return; }
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3 + _lfoOutAM_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _procE(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				x = (p * _lfoMod_0) >> 11;
				_lfoOutAM_0 = x & 0x7ffffffe;
				x = (p * _lfoMod_1) >> 11;
				_lfoOutAM_1 = x & 0x7ffffffe;
				x = (p * _lfoMod_2) >> 11;
				_lfoOutAM_2 = x & 0x7ffffffe;
				x = (p * _lfoMod_3) >> 11;
				_lfoOutAM_3 = x & 0x7ffffffe;
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0 + _lfoOutAM_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				x = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[x];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
			// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1 + _lfoOutAM_1;
			// pg update --->
				x = _wt_1[_pgPhase_1 >> _PG_PHASE_FP] + x;
			// OP 1 out ----->
				o += _log2vTI[x];
			// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2 + _lfoOutAM_2;
				// pg update --->
				p = (o + (_pgPhase_2 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_2[p] + x;
				// OP 1 out ----->
				o = _log2vTI[x];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else { killProcess(); return; }
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3 + _lfoOutAM_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				x = _wt_3[p] + x;
				// OP 1 out ----->
				n = _log2vT[x];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
	private function _procF(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, x:int, p:int, f:int, o:int,
			_et_0:Vector.<int> = _egLevelTable_0, _et_1:Vector.<int> = _egLevelTable_1,
			_et_2:Vector.<int> = _egLevelTable_2, _et_3:Vector.<int> = _egLevelTable_3;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
		      _log2vTI:Vector.<int> = _LOG2V_TABLE_I,
			_wt_0:Vector.<int> = _waveTable_0, _wt_1:Vector.<int> = _waveTable_1,
			_wt_2:Vector.<int> = _waveTable_2, _wt_3:Vector.<int> = _waveTable_3;
		for ( ; i < end; i++)
		{
		// lfo Update --->
			if (_lfoTimer <= 0)
			{
				p = _lfoWaveTable[_lfoPhase];
				x = (p * _lfoMod_0) >> 11;
				_lfoOutAM_0 = x & 0x7ffffffe;
				x = (p * _lfoMod_1) >> 11;
				_lfoOutAM_1 = x & 0x7ffffffe;
				x = (p * _lfoMod_2) >> 11;
				_lfoOutAM_2 = x & 0x7ffffffe;
				x = (p * _lfoMod_3) >> 11;
				_lfoOutAM_3 = x & 0x7ffffffe;
				_lfoPhase = _lfoPhase + 2 & _LFO_WAVE_TABLE_MASK;
				_lfoTimer += _LFO_TIMER_INIT_VALUE;
			}
		// Operator 0 --->
			if (state & 1)
			{
				// eg update --->
				if (_egCount_0 >= _egGate_0)
				{
					if (_shiftEGState_0())_et_0 = _egLevelTable_0;
					else
					{
						if (state & 0xe) { state &= 0xfe; _egCount_0 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_0[_egCount_0 >> _EG_CNT_FP] + _egTLv_0 + _lfoOutAM_0;
				// pg update --->
				p = _pgPhase_0 >> _PG_PHASE_FP;
				// FeedBack -->
				f = _log2vTI[_wt_0[p] + _fdbk];
				// OP 0 out --->
				p = _wt_0[p + f & _WAVE_TABLE_MASK] + x;
				o = _log2vTI[p];
				// counter --->	
				_egCount_0 += _egCountStep_0;
				_pgPhase_0 = _pgPhase_0 + _pgPhaseStep_0 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 1 --->
			if (state & 2)
			{
			// eg update --->
				if (_egCount_1 >= _egGate_1)
				{
					if (_shiftEGState_1())_et_1 = _egLevelTable_1;
					else
					{
						if (state & 0xd) { state &= 0xfd; _egCount_1 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_1[_egCount_1 >> _EG_CNT_FP] + _egTLv_1 + _lfoOutAM_1;
			// pg update --->
				p = (o + (_pgPhase_1 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				p = _wt_1[p] + x;
			// OP 1 out ----->
				o = _log2vTI[p];
			// counter --->
				_egCount_1 += _egCountStep_1;
				_pgPhase_1 = _pgPhase_1 + _pgPhaseStep_1 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 2 --->
			if (state & 4)
			{
				// eg update --->
				if (_egCount_2 >= _egGate_2)
				{
					if (_shiftEGState_2())_et_2 = _egLevelTable_2;
					else
					{
						if (state & 0xb) { state &= 0xfb; _egCount_2 = _EG_MAX_CNT - 1; }
						else { killProcess(); return; }
					}
				}
				x = _et_2[_egCount_2 >> _EG_CNT_FP] + _egTLv_2 + _lfoOutAM_2;
				// pg update --->
				p = (o + (_pgPhase_2 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				p = _wt_2[p] + x;
				// OP 1 out ----->
				o = _log2vTI[p];
				// counter --->
				_egCount_2 += _egCountStep_2;
				_pgPhase_2 = _pgPhase_2 + _pgPhaseStep_2 & _PG_PHASE_MASK;
			}
			else o = 0;
		// Operator 3 --->
			if (state & 8)
			{
				// eg update --->
				if (_egCount_3 >= _egGate_3)
				{
					if (_shiftEGState_3())_et_3 = _egLevelTable_3;
					else { killProcess(); return; }
				}
				x = _et_3[_egCount_3 >> _EG_CNT_FP] + _egTLv_3 + _lfoOutAM_3;
				// pg update --->
				p = (o + (_pgPhase_3 >> _PG_PHASE_FP)) & _WAVE_TABLE_MASK;
				p = _wt_3[p] + x;
				// OP 1 out ----->
				n = _log2vT[p];
				// counter --->
				_egCount_3 += _egCountStep_3;
				_pgPhase_3 = _pgPhase_3 + _pgPhaseStep_3 & _PG_PHASE_MASK;
			}
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_lfoTimer -= _lfoTimerStep;
		}
		startFrame = i;
	}
// process --- Perc
	// Sampler 1ch
	private function _procSmpl1ch(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, n:Number, v:int,
			_et:Vector.<int> = _egLevelTable_0;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
			_wt:Vector.<int> = _waveTable_0;
		for ( ; i < end; i++)
		{
		// eg update --->
			if (_egCount_0 >= _egGate_0)
			{
				if (_shiftEGState_0())_et = _egLevelTable_0;
				else { killProcess(); return; }
			}
			v = _et[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;	
		// pg update --->
			if (_pgPhase_0 >= _loopE)
			{
				if (_loopF > 0) do{ _pgPhase_0 -= _loopF; } while (_pgPhase_0 >= _loopE);
				else { killProcess(); return; }
			}
			v = _wt[_pgPhase_0] + v;
			n = _log2vT[v];
		// out --->
			outL[i] += n * _vOutL;
			outR[i] += n * _vOutR;
		// counter --->
			_egCount_0 += _egCountStep_0;
			_pgPhaseAFP += _pgPhaseStep_0;
			_pgPhase_0 += _pgPhaseAFP >> _PG_PHASE_FP;
			_pgPhaseAFP &= _PG_PHASE_FP_MASK;
		}
		startFrame = i;
	}
	// Sampler 2ch
	private function _procSmpl2ch(outL:Vector.<Number>, outR:Vector.<Number>, end:int):void
	{
		var i:int = startFrame, v:int, p:int, 
			_et:Vector.<int> = _egLevelTable_0;
		const _log2vT:Vector.<Number> = _LOG2V_TABLE, 
			_wt:Vector.<int> = _waveTable_0;
		for ( ; i < end; i++)
		{
		// eg update --->
			if (_egCount_0 >= _egGate_0)
			{
				if (_shiftEGState_0())_et = _egLevelTable_0;
				else { killProcess(); return; }
			}
			v = _et[_egCount_0 >> _EG_CNT_FP] + _egTLv_0;	
		// pg update --->
			if (_pgPhase_0 >= _loopE)
			{
				if (_loopF > 0) do{ _pgPhase_0 -= _loopF; } while (_pgPhase_0 >= _loopE);
				else { killProcess(); return; }
			}
		// out --->
			p = _pgPhase_0 << 1;
			outL[i] += _log2vT[_wt[p    ] + v] * _vOutL;
			outR[i] += _log2vT[_wt[p + 1] + v] * _vOutR;
		// counter --->
			_egCount_0 += _egCountStep_0;
			_pgPhaseAFP += _pgPhaseStep_0;
			_pgPhase_0 += _pgPhaseAFP >> _PG_PHASE_FP;
			_pgPhaseAFP &= _PG_PHASE_FP_MASK;
		}
		startFrame = i;
	}
}
}