/*! ********************************************
 * SimpleSequencer.as
 * Copyright 2022 masahiro takamoto (MotChan)
 * Distributed under the MIT license. 
 * see -> license.txt
 ******************************************** */
package {
import sora.graybox.GrayBox;
import sora.graybox.IGrayBoxModule;
import sora.graybox.MidiEvent;
import sora.graybox.ProcessData;
// シーケンサーモジュール
public class SimpleSequencer implements IGrayBoxModule {
	private var _note:int = 72;  // ノートナンバー
	private var _frameCount:int = 0;  // フレームカウンター
	private var _sampleRate:int = 44100;  // サンプリング周波数
	
	public function SimpleSequencer():void {}
// sora.graybox.IGrayBoxModule -->
	// ホストのサンプリングレートが変更された時に呼ばれるメソッド
	// GrayBox.setSampleRate() -> このメソッド
	public function updateSampleRate(sampleRate:Number):void {
		_sampleRate = int(sampleRate);
		if (_frameCount >= _sampleRate)_frameCount = 0;
	}
	// ホストのオーディオチャンネル数が変更された時に呼ばれるメソッド
	// GrayBox.setAudioChannels() -> このメソッド
	public function updateAudioChannels(ch:int):void{}
	// ホストのプロセスブロックサイズが変更された時に呼ばれるメソッド
	// GrayBox.setProcessBlockSize() -> このメソッド
	public function updateProcessBlockSize(size:int):void{}
	// ホストのGrayBox.resetModules()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.resetModules() -> このメソッド
	public function resetModule():void{ _frameCount = 0; _note = 72; }
	// ホストのGrayBox.initiModules()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.initiModules() -> このメソッド
	public function initiModule():void{ _frameCount = 0; _note = 72; }
	// このモジュールがホストへ接続された時に呼ばれるメソッド
	// GrayBox.attachModule() -> このメソッド
	public function connectModule(host:GrayBox):void {
		updateSampleRate(host.getSampleRate());
	}
	// このモジュールがホストから取り外された時に呼ばれるメソッド
	// GrayBox.detachModule() -> このメソッド
	public function disconnectModule():void{}
	// このモジュールが接続されているスロットの、バイパス状況が更新された時に呼ばれるメソッド
	// GrayBox.bypassProcess() -> このメソッド
	public function updateBypassStatus(bypass:Boolean):void{}
	// ホストのGrayBox.startProcess()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.startProcess() -> このメソッド
	public function startProcess():void{}
	// ホストのGrayBox.stopProcess()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.stopProcess() -> このメソッド
	public function stopProcess():void{}
	// ホストのGrayBox.suspendProcess()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.suspendProcess() -> このメソッド
	public function suspendProcess():void{}
	// ホストのGrayBox.resumeProcess()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.resumeProcess() -> このメソッド
	public function resumeProcess():void{}
	// ホストが音声処理を実行した時に呼ばれるメソッド
	// GrayBox.process() -> このメソッド
	public function process(procData:ProcessData):void {
		// ProcessDataオブジェクトはホストから音声処理に必要な情報を受け取り、
		// 各モジュールへ伝えるオブジェクトです。
		// GrayBoxシステムにおけるMIDIメッセージの伝送はProcessDataオブジェクトが持つ
		// MIDIイベントリスト(片方向リンクリスト)を使用して行われます。
		// MIDIメッセージの送信側はこのリストにMIDIイベントを登録していき、
		// MIDIメッセージの受信側はこのリストからMIDIイベント参照してメッセージの処理を行います。
		
		// ・ProcessData.midiEventList
		//   MIDIイベントリスト(先頭ノード(リスト管理用ダミーノード))
		//   このリストのノードの並びはMidiEvent.offsetFramesの昇順です。
		// ・ProcessData.addToMidiEventList(event:MidiEvent, check:Boolean = false)
		//   MIDIイベントリストへイベントを登録します。(通常はこちらを使用してください)
		//   第1引数 リストに登録するMidiEventオブジェクト
		//   第2引数 登録しようとするMidiEventオブジェクトが
		//           リストに既に登録されているかをチェックする
	// MIDIイベント生成、登録 -->
		var _frame:int = procData.offsetFrames, // 開始フレーム
		    _frameEnd:int = _frame + procData.sampleFrames; // 終了フレーム
		for ( ; _frame < _frameEnd; _frame++) {
			// 1秒ごとにMIDIイベント生成 -->
			if (_frameCount++ >= _sampleRate) {
			// ノートオフメッセージ (前回の分)
				var _midiEvent:MidiEvent = new MidiEvent();  // MIDIイベント生成
				_midiEvent.type = 0x80;  // ノートオフ
				_midiEvent.channel = 0;  // Midiチャンネル
				_midiEvent.value0 = _note;  // ノートナンバー
				_midiEvent.offsetFrames = _frame; // このイベントの発生フレーム
				procData.addToMidiEventList(_midiEvent); // MIDIイベントリストへ登録
			// ノートナンバー更新
				if (++_note > 72)_note = 60;
			// ノートオンメッセージ (今回の分)
				_midiEvent = new MidiEvent();  // MIDIイベント生成
				_midiEvent.type = 0x90; // ノートオン
				_midiEvent.channel = 0; // Midiチャンネル
				_midiEvent.value0 = _note; // ノートナンバー
				_midiEvent.value1 = 127; // ノートオンベロシティー
				_midiEvent.offsetFrames = _frame; // このイベントの発生フレーム
				procData.addToMidiEventList(_midiEvent); // MIDIイベントリストへ登録
				// フレームカウンターリセット
				_frameCount = 0;
			}
		}
	}
}
}